from __future__ import print_function
from __future__ import absolute_import
import psutil
import os
import platform
import re
import netifaces
import logging
import inspect
from .common import *
import sillyfacter.config


MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


def fetch():
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering network info...")
    nwio = psutil.net_io_counters(pernic=True)
    nw = []
    for i in netifaces.interfaces():
        try:
            ip = netifaces.ifaddresses(i)[2][0]['addr']
            recv = nwio[i].bytes_recv
            sent = nwio[i].bytes_sent
            nw.append({"ip": ip, "recv": recv, "sent": sent, "ifname": i})
        except:
            if sillyfacter.config.STRICT:
                raise
            if sillyfacter.config.DEBUG:
                l.exception("")
            ip = None
            recv = nwio[i].bytes_recv
            sent = nwio[i].bytes_sent
            nw.append({"ip": ip, "recv": recv, "sent": sent, "ifname": i})
        l.debug("Adding interface '{}' with ip {}".format(i, str(ip),))
    return nw



from __future__ import print_function
from __future__ import absolute_import
import psutil
import os
import platform
import re
import logging
import inspect
import datetime
from .common import *


MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


def fetch():
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering users info...")
    users = {}
    users["current"] = current_users()
    users["last"] = last_users()
    return users


def current_users():
    ldict = {"step": inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering logged in users...")
    host_users = []
    for item in psutil.get_users():
        user = item[0]
        term = item[1]
        host = item[2]
        host_users.append({"user": user,
                           "terminal": term,
                           "from": host})
    return host_users


def last_users(last=20):
    ldict = {"step": inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    l.info("Gather users logged in 20 days")
    ret = []
    ostype = os.uname()[0]
    if ostype == "SunOS":
        last_cmd = ["/bin/last", "-a"]
    elif ostype == "Linux":
        last_cmd = ["/usr/bin/last", "-a"]
    else:
        last_cmd = ["/usr/bin/last"]
    last = Command(last_cmd)
    if last.run(maxtime=5) is not None:
        output = last.output
        ret = last_parser(output, last=last)
    return ret


def last_parser(lastout, last=20):
    _log_level = "not_debug"
    dprint = debugprint(log_level=_log_level)
    last_users = []
    today = datetime.datetime.today()
    re_last = '([0-9a-z]+)\s+' + \
              '.+' + \
              '(Mon|Tue|Wed|Thu|Fri|Sat|Sun)\s+' + \
              '([a-z]{3})\s+' + \
              '([0-9]+)\s+' + \
              '([0-9][0-9]):([0-9][0-9]).*$'
    re_last = re.compile(re_last, re.IGNORECASE)
    regx = re_last
    for line in lastout:
        dmsg = dprint()
        line = str(line.strip())
        dmsg += ["Input: '{}'".format(line)]
        if re.search('still logged in', line):
            continue
        m = regx.match(line)
        if m:
            user = m.group(1).strip()
            mm = m.group(3).strip()
            dd = int(m.group(4).strip())
            hour = m.group(5).strip()
            mins = m.group(6).strip()
            date_string = "{:02} {} {} {} {}".format(dd, mm, today.year,
                                                     hour, mins)
            date_format = "%d %b %Y %H %M"
            parsed_date = datetime.datetime.strptime(date_string,
                                                     date_format)
            if parsed_date > today:
                date_string = "{:02} {} {} {} {}".format(dd, mm,
                                                         today.year - 1,
                                                         hour, mins)
                parsed_date = datetime.datetime.strptime(date_string,
                                                         date_format)
            if parsed_date > (today - datetime.timedelta(days=20)):
                last_users.append(user)
            dmsg.append("set: last_users.add({})".format(user))

        else:
            dmsg.append("match: Bad match")
        dmsg = dprint(dmsg, "lastuser")
        last_users = list(set(last_users))
    return last_users
    #



from __future__ import print_function
from __future__ import absolute_import
import os
import re
import logging
import platform
import inspect
import json
import datetime
import psutil
from ..common import *

MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            encoded_object = obj.strftime('%s')
        else:
            encoded_object = json.JSONEncoder.default(self, obj)
        return encoded_object


def output(modules=[], out=None):
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering output...")
    fetched, fetchable = fetchprocessor(modules)
    retstr, retval = None, None
    fetchraw = None
    try:
        fetchraw = fetch2json(fetched,
                              fetchable)
        jsonstr = json.dumps(fetchraw,
                             sort_keys=True,
                             indent=4,
                             cls=DateTimeEncoder,
                             separators=(',', ': '))
    except:
        l.exception("Unable to convert obj to JSON")
        l.warning("RAW:\n{}".format(fetchraw))
        retstr = ""
        retval = 1
    else:
        retstr = jsonstr
        retval = 0
    return retstr, retval


def lookup(f, t=None):
    if "filesystem" in f:
        fs_list = f["filesystem"]
    else:
        fs_list = {}
    if "nslookup" in f:
        ns_cache = f["nslookup"]
    else:
        ns_cache = {}

    def _fslookup(dev=None, mount=None):
        retval = (None, None)
        if dev is not None:
            key, val, get = "dev", dev, "mount"
        elif mount is not None:
            key, val, get = "mount", mount, "dev"
        else:
            return retval
        for fs in fs_list:
            if fs[key] and fs[key] == val:
                if fs[get]:
                    retval = (fs[get], fs["fstype"])
                    break
        return retval

    def _nslookup(ip=None):
        retval = None
        if ip is not None:
            retval = ip
            if ip not in ns_cache:
                ns_cache[ip] = fetch_dns(ip)
            retval = ns_cache[ip]
        return retval

    if t is None:
        return _fslookup, _nslookup
    elif t == "fslookup":
        return _fslookup
    else:
        return _nslookup


def fetch2json(f, fetchable):
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    myfslookup, mynslookup = lookup(f)
    fetched = {}
    fetched["_scan_id"] = platform.node()
    fetched["_scan_time"] = datetime.datetime.now()
    fetched["has"] = {}
    fetched["had"] = {}
    dt = datetime.datetime.fromtimestamp
    boottime = dt(psutil.get_boot_time())
    # host main
    if "os" in f:
        for item, val in f["os"].iteritems():
            fetched[item] = val

    # special
    if "custom" in f:
        pass

    # host networks
    if "network" in f:
        fetched["has"]["network"] = []
        for item in f["network"]:
            newitem = {}
            newitem["ifname"] = item["ifname"]
            newitem["ip"] = item["ip"]
            #if newitem["ip"] in f["nslookup"]:
            #    newitem["hostname"] = f["nslookup"][newitem["ip"]]
            fetched["has"]["network"].append(newitem)

    # host procs
    if "process" in f:
        fetched["has"]["process"] = []
        for pid in f["process"]:
            _process_type = set()
            newitem = {}
            newitem["pid"] = pid
            newitem["commandline"] = f["process"][pid]["cmdline"]
            newitem["exe"] = f["process"][pid]["exe"]
            if "exe_fs" in f["process"][pid]:
                newitem["exe_fs"] = f["process"][pid]["exe_fs"]
            newitem["user"] = f["process"][pid]["user"]
            newitem["createtime"] = f["process"][pid]["create_time"]
            if boottime + datetime.timedelta(seconds=600) \
                    > newitem["createtime"]:
                _process_type.add("startup-prog")
            else:
                _process_type.add("user-prog")
            #
            # proc network conn
            if "connection" in f["process"][pid]:
                newitem["connection"] = []
                newconns = f["process"][pid]["connection"]
                for conn in newconns:
                    conn_src = conn[0]
                    conn_src_ip = ":".join(conn_src.split(':')[:-1])
                    conn_src_port = ":".join(conn_src.split(':')[-1:])
                    conn_src_host = conn_src_ip
                    conn_dst = conn[1]
                    if conn_dst is not None:
                        conn_dst_ip = ":".join(conn_dst.split(':')[:-1])
                        conn_dst_port = ":".join(conn_dst.split(':')[-1:])
                        conn_dst_host = conn_dst_ip
                        _process_type_network = "network-client"
                    else:
                        conn_dst_port = "*"
                        conn_dst_ip = "0.0.0.0"
                        conn_dst_host = "*"
                        _process_type_network = "network-daemon"
                    conn_pair = {"source": conn_src_host + ":" + conn_src_port,
                                 "destination": conn_dst_host +
                                 ":" + conn_dst_port}
                    newitem["connection"].append(conn_pair)
                    _process_type.add(_process_type_network)
                if len(newitem["connection"]) == 0:
                    del newitem["connection"]
            #
            # proc open mounts
            if "open" in f["process"][pid]:
                newitem["open"] = {"filesystem": []}
                open_fs = set()
                mountpairs = f["process"][pid]["open"]
                for mountpair in mountpairs:
                    if "fs" in mountpair:
                        fs = mountpair["fs"]
                        open_fs.add(fs)
                        _, fstype = myfslookup(dev=fs)
                        if fstype is not None and len(fstype) > 0:
                            _process_type.add("{}-client".format(fstype))
                open_fs_list = list(open_fs)
                if len(open_fs_list) > 0:
                    newitem["open"]["filesystem"] = list(open_fs)
                else:
                    del newitem["open"]
            newitem["type"] = list(_process_type)
            fetched["has"]["process"].append(newitem)
    if "user" in f:
        users = f["user"]
        if "current" in users:
            fetched["has"]["user"] = users["current"]
        if "last" in users:
            fetched["had"]["user"] = users["last"]
    #
    if "custom" in f:
        fetched_custom = {}
        for modulename, val in f["custom"].iteritems():
            try:
                fetched_custom = None
                m = fetchable["custom"][modulename]
                if hasattr(m, "output_for_json"):
                    fetched_custom = getattr(m, "output_for_json")(val)
                elif hasattr(m, "output"):
                    fetched_custom = getattr(m, "output")(val)
                else:
                    l.error("Could not find outout handler in module " +
                            "'{}'".format(m))
            except:
                l.error("custom module {} not loaded".format(modulename))
            else:
                if fetched_custom is not None:
                    fetched = dict(fetched, **fetched_custom)
    return fetched



from __future__ import print_function
from __future__ import absolute_import
import os
import re
import subprocess
import threading
import logging
import socket
import inspect
import tempfile
import platform
import stat
import shlex
import sillyfacter.config
#from common import *

MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))
DEBUG = sillyfacter.config.DEBUG
STRICT = sillyfacter.config.STRICT


def debugprint(log_level=None):
    def dprint(lines=[], module=""):
        if type(lines) is not list:
            lines = [lines]
        if len(module) != 0:
            module = "[{}]".format(module)
        if len(lines) == 0:
            print("{}:".format(log_level), module, "-" * 40)
        else:
            for line in lines:
                print("{}:".format(log_level), module, line)
        return []

    def noop(a=[], b=""):
        return []

    if log_level == "DEBUG" or sillyfacter.config.DEBUG:
        return dprint
    else:
        return noop


def timeout(func, args=(), kwargs={},
            timeout_duration=10, default=None, log=None):
    """This function will spawn a thread and run the given function
    using the args, kwargs and return the given default value if the
    timeout_duration is exceeded.
    """
    class InterruptableThread(threading.Thread):

        def __init__(self):
            threading.Thread.__init__(self)
            self.result = default

        def run(self):
            self.result = func(*args, **kwargs)
    try:
        if log:
            log.info("Starting tiemoutthread for '{}' timeout in {}s".format(
                func.__name__, timeout_duration))
        it = InterruptableThread()
        it.start()
        it.join(timeout_duration)
        if it.isAlive():
            return it.result
        else:
            return it.result
    except:
        if log:
            log.warning("Exception occurred in timerthread for '{}'".format(
                func.__name__))
        return default


def _execute(cmd, stdout=None, stderr=None):
    retval = None
    if stderr is None:
        stderr = stdout
    try:
        proc = subprocess.Popen(cmd,
                                universal_newlines=True,
                                stdout=stdout,
                                stderr=stderr)
        retval = proc.wait()
    except Exception as _:
        if STRICT:
            raise
        if DEBUG:
            print("Exception has occurred in _execute(cmd, stdout, stderr)")
            dp = debugprint()
            dp("Error in executing '{}' exception is '{}'".
               format(cmd, _), "_execute")
        pass
    return retval


def _read_unlink_handle(fh, fn):
    retval = []
    try:
        with os.fdopen(fh) as fd:
            fd.seek(0)
            retval = fd.readlines()
    except:
        if STRICT:
            raise
        pass
    finally:
        try:
            os.unlink(fn)
        except:
            pass
    return retval


class Command(object):
    def __init__(self, cmd, log=False):
        self.cmd = cmd
        self.process = None
        self.output = []
        self.errput = []
        self.errfile_handle, self.errfile_name = tempfile.mkstemp()
        self.outfile_handle, self.outfile_name = tempfile.mkstemp()

    def run(self, maxtime=30):
        self.retval = timeout(_execute,
                              args=(self.cmd,
                                    self.outfile_handle,
                                    self.errfile_handle),
                              timeout_duration=maxtime)
        self.output = _read_unlink_handle(self.outfile_handle,
                                          self.outfile_name)

        self.errput = _read_unlink_handle(self.errfile_handle,
                                          self.errfile_name)
        return self.retval


def run_command(args):
    if isinstance(args, basestring):
        args = shlex.split(args)
    try:
        prg = Command(args)
        prg.run()
    except Exception as _:
        if STRICT:
            raise
        if DEBUG:
            dp = debugprint()
            dp("Error in executing '{}' exception is '{}'".
               format(args, _), "run_command")
        rc = 1
        out = "UnknownError"
        err = "UnknownError"
    else:
        rc, out, err = prg.retval, ''.join(prg.output), ''.join(prg.errput)
    return (rc, out, err)


def is_executable(path):
    '''is the given path executable?'''
    return (stat.S_IXUSR & os.stat(path)[stat.ST_MODE]
            or stat.S_IXGRP & os.stat(path)[stat.ST_MODE]
            or stat.S_IXOTH & os.stat(path)[stat.ST_MODE])


def get_bin_path(arg, required=False, opt_dirs=[]):
    '''
    find system executable in PATH.
    Optional arguments:
       - required:  if executable is not found and required is true, fail_json
       - opt_dirs:  optional list of directories to search in addition to PATH
    if found return full path; otherwise return None
    '''
    sbin_paths = ['/sbin', '/usr/sbin', '/usr/local/sbin']
    paths = []
    for d in opt_dirs:
        if d is not None and os.path.exists(d):
            paths.append(d)
    paths += os.environ.get('PATH', '').split(os.pathsep)
    bin_path = None
    # mangle PATH to include /sbin dirs
    for p in sbin_paths:
        if p not in paths and os.path.exists(p):
            paths.append(p)
    for d in paths:
        path = os.path.join(d, arg)
        if os.path.exists(path) and is_executable(path):
            bin_path = path
            break
    return bin_path


def pretty_bytes(size=0):
    ranges = ((1 << 70L, 'ZB'),
              (1 << 60L, 'EB'),
              (1 << 50L, 'PB'),
              (1 << 40L, 'TB'),
              (1 << 30L, 'GB'),
              (1 << 20L, 'MB'),
              (1 << 10L, 'KB'),
              (1, 'Bytes'))
    for limit, suffix in ranges:
        if size >= limit:
            break
    return '%.2f %s' % (float(size)/limit, suffix)


def fetch_dns(ip="127.0.0.1"):
    try:
        name = socket.gethostbyaddr(str(ip))[0]
    except:
        if re.match('^127\.[.0-9]*$', ip):
            name = "localhost"
        else:
            name = ip
    return name


def importer(modules=[]):
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    fetchable = {"custom": {}}
    for m in modules:
        try:
            fetchable["custom"][m] = \
                __import__("sillyfacter.custom.{}".format(m),
                           fromlist=[m],
                           level=0)
        except Exception as _:
            l.info("No custon module 'custom/{}' available".format(m))
            try:
                fetchable[m] = __import__("sillyfacter.{}".format(m),
                                          fromlist=[m],
                                          level=0)
            except Exception as _:
                if STRICT:
                    raise
                l.exception("Import exception for module " +
                            "'{}': {}".format(m, _))
                l.error("Import failure for module '{}'".format(m))
            else:
                l.info("Import success for module '{}'".format(m))
        else:
            l.info("Import success for  module 'custom/{}'".format(m))
    return fetchable


def fetcher(fetchable={}):
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    fetched = {}
    for modulename, moduleobj in fetchable.iteritems():
        if modulename is not "custom":
            try:
                fetched[modulename] = getattr(moduleobj, "fetch")()
            except Exception as _:
                if STRICT:
                    raise
                if DEBUG:
                    l.excetion("fetch failure for module " +
                               "{}".format(modulename))
                else:
                    l.warning("fetch failure for module " +
                              "{}, {}".format(modulename, _))
            else:
                l.info("fetch success for module {}".format(modulename))
        else:
            if "custom" not in fetched:
                fetched["custom"] = {}
            for modulename, moduleobj in fetchable["custom"].iteritems():
                try:
                    fetched["custom"][modulename] = \
                        getattr(moduleobj, "fetch")()
                except Exception as _:
                    if STRICT:
                        raise
                    if DEBUG:
                        l.exception("fetch failure for module " +
                                    "{}".format(modulename))
                    else:
                        l.exception("fetch failure for module " +
                                    "{}, {}".format(modulename, _))
                else:
                    l.info("fetch success for module {}".format(modulename))
    return fetched, fetchable


def fetchprocessor(modules=[]):
    return fetcher(importer(modules))


def fetch_lg(name=None):
    return logging.getLogger("sillyfacter")


def setup_logger(name=None,
                 logfile=None,
                 console=True,
                 level=60,
                 ):
    log = fetch_lg()
    mylevel = 60 - level*20
    if mylevel < 10:
        log.setLevel(logging.DEBUG)
    elif mylevel >= 60:
        console = False
        log.setLevel(logging.CRITICAL)
    else:
        log.setLevel(mylevel)
    #
    if logfile is not None:
        fh = logging.FileHandler(file)
        fh.setLevel(logging.DEBUG)
        fmtf = logging.Formatter(
            '%(asctime)s - %(name)s ' +
            '%(hostname)-16s %(levelname)-8s ' +
            '@%(step)-30s %(message)s')
        fh.setFormatter(fmtf)
        log.addHandler(fh)
    #
    if console is True:
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        fmtc = logging.Formatter(
            '%(asctime)s - %(name)s ' +
            '%(hostname)-16s %(levelname)-8s ' +
            '@%(step)-30s %(message)s')
        ch.setFormatter(fmtc)
        log.addHandler(ch)
    return log



from __future__ import print_function
from __future__ import absolute_import
import os
import re
import logging
import platform
import inspect
import pymongo
import json
from .json import fetch2json
from ..common import *

MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


def output(modules=[], out='mongodb://localhost/sillyfacter_db'):
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    fetched, fetchable = fetchprocessor(modules)
    fetchobj = fetch2json(fetched, fetchable)
    fetchobj["_id"] = fetchobj["_scan_id"]
    return_dict = {"status": "Save failed",
                   "returncode": 1,
                   "url": out}
    coll, db, conn = _collection(dburl=out)
    retstr, returnval = None, None
    if coll is not None:
        try:
            docid = coll.save(fetchobj, safe=True)
            return_dict["status"] = "Save success"
            return_dict["_info"] = {}
            return_dict["_info"]["_id"] = docid
            return_dict["_info"]["collection"] = coll.name
            return_dict["_info"]["db"] = db.name
            return_dict["_info"]["host"] = conn.host
            return_dict["_info"]["port"] = conn.port
        except Exception as ex:
            l.exception("Unable to save data in collection: {}".format(ex))
            retval = 1
        else:
            l.info("Saved data with _id as '{}'".format(docid))
            return_dict["returncode"] = 0
            retval = 0
        finally:
            conn.close()
    retstr = json.dumps(return_dict,
                        sort_keys=True,
                        indent=4,
                        separators=(',', ': '))
    return retstr, retval


def _collection(dburl='mongodb://localhost/sillyfacter_db'):
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    if dburl is not None:
        pass
    else:
        dburl = 'mongodb://localhost/sillyfacter_db'
    try:
        conn = pymongo.MongoClient(dburl)
    except Exception as ex:
        l.exception("Connection to '{}' failed: {}".format(dburl, ex))
        coll = None
        db = None
        conn = None
    else:
        try:
            db = conn.get_default_database()
        except:
            db = conn["sillyfacter_db"]
        coll = db["sillyfacter"]
    return coll, db, conn



from __future__ import print_function
from __future__ import absolute_import
import psutil
import platform
import logging
import inspect
import os
import re
import datetime
from .common import *
import sillyfacter.config


MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


def fetch():
    #
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering host info...")
    host_props = {}
    host_props["machine"] = platform.machine()
    host_props["arch"] = platform.machine()
    host_props["release"] = platform.release()
    host_props["system"] = platform.system()
    host_props["hostname"] = platform.node()
    host_props["name"] = host_props["hostname"]
    host_props["domainname"] = ".".join(socket.gethostname().split('.')[1:])
    host_props["fqdn"] = socket.gethostname()
    host_props["memory"] = psutil.virtual_memory().total
    host_props["memory_used"] = psutil.virtual_memory().percent
    host_props["cpu"] = psutil.NUM_CPUS
    dt = datetime.datetime.fromtimestamp
    host_props["boottime"] = dt(psutil.get_boot_time())
    host_props["cpu_used"] = psutil.cpu_percent(interval=1, percpu=False)
    host_props["hardwareisa"] = platform.processor()
    host_props["hardwaremodel"] = platform.processor()
    host_props["user"] = os.environ['USER']
    ostype = os.uname()[0]
    if ostype == "Linux":
        host_props["kernel_release"] = platform.uname()[2]
        host_props["kernel_version"] = platform.uname()[2].split('-')[0]
        try:
            host_props_extra = fetch_linux()
        except Exception as _:
            if sillyfacter.config.STRICT:
                raise
            if sillyfacter.config.DEBUG:
                l.exception("")
            l.error("unable to fetch linux specific info '{}'".format(_))
            host_props_extra = {}
        host_props = dict(host_props.items() + host_props_extra.items())
    for i in host_props:
        l.debug("{:<15}: {}".format(i, host_props[i]))
    return host_props


def _run_this(c):
    cmd = Command(c)
    retval = None
    if cmd.run() == 0:
        return cmd.output
    else:
        return retval


def _open_read_this(filepath):
    retval = ""
    with open(filepath, 'r') as f:
        retval = f.read()
        retval.strip()
    return retval


def run_parse_lsb():
    lsb_finder = _run_this(["/usr/bin/which", "lsb_release"])
    retval = {}
    if lsb_finder is not None:
        lsbcmd = _run_this([lsb_finder[0].strip(), "-a"])
        if lsbcmd is not None:
            lsb_parsed = {}
            for line in lsbcmd:
                line = line.strip()
                key = line.split(':')[0]
                val = ":".join(line.split(':')[1:]).strip()
                key = key.lower().replace('lsb', '')
                key = key.strip().replace(' ', '_')
                lsb_parsed["lsb_{}".format(key)] = val
            retval = lsb_parsed
    return retval


## Similar to Phacter :-)
def fetch_linux():
    fetch = run_parse_lsb()
    if fetch["lsb_distributor_id"] == 'Ubuntu':
        operatingsystem = "Ubuntu"
        operatingsystemrelease = fetch["lsb_release"]
    elif os.path.isfile('/etc/debian_version'):
        operatingsystem = 'Debian'
        operatingsystemrelease = _open_read_this('/etc/debian_version')
    elif os.path.isfile('/etc/gentoo-release'):
        operatingsystem = 'Gentoo'
        operatingsystemrelease = _open_read_this('/etc/gentoo-release')
    elif os.path.isfile('/etc/fedora-release'):
        operatingsystem = 'Fedora'
        operatingsystemrelease = _open_read_this('/etc/fedora-release')
    elif os.path.isfile('/etc/mandriva-release'):
        operatingsystem = 'Mandriva'
        operatingsystemrelease = _open_read_this('/etc/mandriva-release')
    elif os.path.isfile('/etc/mandrake-release'):
        operatingsystem = 'Mandrake'
        operatingsystemrelease = _open_read_this('/etc/mandrake-release')
    elif os.path.isfile('/etc/redhat-release'):
        operatingsystemrelease = _open_read_this('/etc/redhat-release')
        if re.compile('centos', re.IGNORECASE).search(operatingsystemrelease):
            operatingsystem = 'CentOS'
        else:
            operatingsystem = 'RedHat'
    elif os.path.isfile('/etc/SuSE-release'):
        operatingsystemrelease = _open_read_this('/etc/SuSE-release')
        if re.compile('SUSE LINUX Enterprise Server',
                      re.I).search(operatingsystemrelease):
            operatingsystem = 'SLES'
        else:
            operatingsystem = 'SuSE'
    else:
        operatingsystem = "Unknown"
        operatingsystemrelease = "Unknown"
    fetch["operatingsystem"] = operatingsystem.strip()
    fetch["operatingsystemreleasestring"] = operatingsystemrelease.strip()
    return fetch



from setuptools import setup, find_packages

import sys, os
import time

name = 'sillyfacter'
setup(name=name,
      version=open("{}/config/__init__.py".
                   format(name)).read().split('VERSION')[1].split('\'')[1],
      description="""\
Sillyfacter prints JSON facts related to the state of the system.
The state here means process running on the host, their open connections
and files. Also users logged in, mount points on the host etcetra.
This information is useful in mapping out dependencies.""",
      long_description=open('README.txt').read(),
      classifiers=[],
      # Get strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='facter json process lsof mounts facts host os state',
      author='Gautam R Singh',
      author_email='g@ut.am',
      url='https://github.com/hihellobolke/Sillyfacter',
      license='Apache',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      scripts=['bin/sillyfacter'],
      install_requires=[
          "psutil >= 1.2.1",
          "netifaces >= 0.8",
          "netaddr >= 0.7.10",
          "pymongo",
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from __future__ import print_function
from __future__ import absolute_import
import psutil
import os
import platform
import re
import logging
import inspect
from .common import *
import sillyfacter.config


MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


def fetch():
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering filesystem info...")
    fs = []
    for p in psutil.disk_partitions(all=False):
        l.info("Found {} mounted as {}".format(p.device, p.mountpoint))
        partition = {}
        partition = {'dev': p.device,
                     "mount": p.mountpoint,
                     "fstype": p.fstype}
        if (re.match('ext[0-9]|xfs|ufs|btrfs', p.fstype, re.IGNORECASE)):
            try:
                disk_usage = psutil.disk_usage(p.mountpoint)
            except:
                if sillyfacter.config.STRICT:
                    raise
                if sillyfacter.config.DEBUG:
                    l.exception("")
            else:
                partition["total"] = disk_usage.total
                partition["used"] = disk_usage.used
                partition["free"] = disk_usage.free
                partition["percent"] = disk_usage.percent
        fs.append(partition)
    for p in psutil.disk_partitions(all=True):
        if (re.match('nfs|autofs', p.fstype, re.IGNORECASE)):
            l.info("Found '{}' mounted as '{}'".format(p.device, p.mountpoint))
            fs.append(
                {'dev': p.device, "mount": p.mountpoint, "fstype": p.fstype})
    return fs



from __future__ import print_function
from __future__ import absolute_import
import psutil
import os
import platform
import re
import netaddr
import logging
import inspect
import datetime
import string
import stat
from collections import deque
from .common import *
import sillyfacter.config


MODULEFILE = re.sub('\.py', '',
                            os.path.basename(inspect.stack()[0][1]))


def fetch():
    ldict = {"step": MODULEFILE + "/" + inspect.stack()[0][3],
             "hostname": platform.node().split(".")[0]}
    l = logging.LoggerAdapter(fetch_lg(), ldict)
    #
    l.info("Gathering process info...")
    allps = {}
    openfiles = {}
    nslookup = {}
    dt = datetime.datetime.fromtimestamp
    for p in psutil.process_iter():
        try:
            if (len(p.cmdline) > 0):
                myps = {
                    "exe": ''.join(s for s in p.exe if s in string.printable),
                    "status": str(p.status),
                    "cmdline": " ".join(p.cmdline),
                    "user": p.username,
                    "memory_percent": p.get_memory_percent(),
                    "create_time": dt(int(p.create_time)),
                    "uid": p.uids.effective,
                    "gids": p.gids.effective,
                    "fd_count": p.get_num_fds(),
                    "thread_count": p.get_num_threads(),
                }
                try:
                    o = sorted(set(
                               [ofile.path for ofile in p.get_open_files()]))
                    openfiles[p.pid] = o
                    myps["open_file_count"] = len(o)
                    myps["open"] = []
                    myps["open_files"] = set()
                    myps["open_fs"] = []
                except:
                    openfiles[p.pid] = []
                    myps["open_file_count"] = 0
                    myps["open"] = []
                    myps["open_files"] = set()
                    myps["open_fs"] = []
                try:
                    c, nslookup = sanitize_conns(
                        [[conn.local_address, conn.remote_address]
                            for conn in p.get_connections()], nslookup)
                    myps["connection"] = c
                except:
                    myps["connection"] = []
                try:
                    i = p.get_io_counters()
                    myps["read_count"] = i.read_count
                    myps["write_count"] = i.write_count
                    myps["read_bytes"] = i.read_bytes
                    myps["write_bytes"] = i.write_bytes
                except:
                    myps["read_count"] = 0
                    myps["write_count"] = 0
                    myps["read_bytes"] = 0
                    myps["write_bytes"] = 0
                l.info(("Processed pid: {} '{}' has " +
                        "open {} files, {} connections"
                        ).format(p.pid,
                                 myps["cmdline"][:25],
                                 myps["open_file_count"],
                                 len(myps["connection"]),
                                 ))
                allps[p.pid] = myps

        except:
            if sillyfacter.config.DEBUG:
                l.exception("")
    mountcache = {}
    execache = {}
    for pid in sorted(allps.keys()):
        """if (allps[pid]["status"] == str(psutil.STATUS_RUNNING) or
            allps[pid]["status"] == str(psutil.STATUS_SLEEPING) or
            allps[pid]["status"] == str(psutil.STATUS_STOPPED) or
            allps[pid]["status"] == str(psutil.STATUS_TRACING_STOP) or
            allps[pid]["status"] == str(psutil.STATUS_WAKING)) and \
           (allps[pid]["open_file_count"] > 0):
            pass
        """
        for f in openfiles[pid]:
            mountcache[f] = None
        exe = allps[pid]["exe"]
        if exe is not None and len(exe) > 0:
            execache[exe] = None
            allps[pid]["exe_fs"] = None
    mountcache = find_file_mounts(mountcache, log=l)
    execache = find_file_mounts(execache, log=l)
    for pid in sorted(allps.keys()):
        p = allps[pid]
        if "exe_fs" in p:
            exe = p["exe"]
            if exe in execache:
                p["exe_fs"] = execache[exe]
        for f in openfiles[pid]:
            mountdevice_set = set()
            openfiles_set = set()
            if f in mountcache:
                mountdevice = mountcache[f]
                if mountdevice is not None:
                    p["open"].append({"path": f, "fs": mountdevice})
                    mountdevice_set.add(mountdevice)
                    openfiles_set.add(f)
            p["open_fs"] = list(mountdevice_set)
            p["open_files"] = list(openfiles_set)
            if "exe" in p and p["exe"] is not None:
                _ = p["open_files"] + [p["exe"]]
                p["open_files"] = list(set(_))
                p["open_file_count"] = len(p["open_files"])
            if "exe_fs" in p and p["exe_fs"] is not None:
                _ = p["open_fs"] + [p["exe_fs"]]
                p["open_fs"] = list(set(_))
            if sillyfacter.config.DEBUG:
                l.debug("Pid: {} Files: {}".format(pid, p["open"]))
    return allps


def df_parser(dfcache, dfout, dferr, filelist):
    # /usr/xpg4/bin/df
    dprint = debugprint(log_level="NODEBUG")
    mycache = {}
    mounts = deque()
    for line in dferr:
        dmsg = dprint()
        line = str(line.strip())
        re_str = '''df: [`(](.+)[')]:? .*'''
        dmsg += ["Input: '{}'".format(line),
                 "Regex: /{}/".format(re_str)]
        m = re.match(re_str, line)

        if m:
            path = m.group(1).strip()
            dmsg.append("match: Good found '{}'".format(path))
            dmsg.append("set: mycache[{}] = '{}'".format(
                        path, "error:/not/found"))
            mycache[path] = "error:/not/found"
        else:
            dmsg.append("match: Bad match")
        dmsg = dprint(dmsg, "dferr")
    #
    for line in dfout:
        dmsg = dprint()
        line = str(line.strip())
        re_str = '''([^ ].*[^ ])\s+[0-9]+''' + \
                 '''\s+[0-9]+\s+[0-9]+\s+([''' + \
                 '''0-9]+[%]|\s[-])\s.*'''
        dmsg += ["Input: '{}'".format(line),
                 "Regex: /{}/".format(re_str)]
        m = re.match(re_str, line)
        if m:
            mount = m.group(1).strip()
            mounts.append(mount)
            dmsg.append("match: Good found '{}'".format(mount))
            dmsg.append("set: mounts.append('{}')".format(mount))
        else:
            dmsg.append("match: Bad match")
        dmsg = dprint(dmsg, "dfout")
    #
    for f in filelist:
        if f not in mycache:
            try:
                mycache[f] = mounts.popleft()
            except:
                raise
                mycache[f] = None
    return dict(dfcache.items() + mycache.items())


def find_file_mounts(files, retry=2, log=None):
    df_cache = {}
    returndict = {}
    for f in files:
        df_cache[f] = None
    filelist = df_cache.keys()
    #
    ostype = os.uname()[0]
    if ostype == "SunOS":
        df_cmd = ["/usr/xpg4/bin/df", "-P"]
    elif ostype == "Linux":
        df_cmd = None
        returndict = _linux_use_stat(filelist, log=log)
    else:
        df_cmd = ["/bin/df", "-P"]
    if df_cmd:
        df_cache = _run_df(filelist, df_cmd)
        for f in files:
            if f in df_cache:
                returndict[f] = df_cache[f]
            else:
                returndict[f] = None
    return returndict


def _run_df(filelist, df_cmd):
    df_cache = {}
    reterr = []
    retout = []
    for filebatch in [filelist[_:_ + 100]
                      for _ in range(0, len(filelist), 100)]:
        df = Command(df_cmd + filebatch)
        if df.run(maxtime=45) is not None:
            retout = df.output
            reterr = df.errput
            df_cache = df_parser(df_cache, retout, reterr, filebatch)
        else:
            for path in filebatch:
                df_cache[path] = None
    return df_cache


def _linux_use_stat(f, log=None):
    devnum_cache = _devicenum_cache(log=log)
    dev_cache = {}
    returndict = {}
    unknown_paths = []
    unknown_cache = {}
    for path in f:
        try:
            blk = os.stat(path)[stat.ST_DEV]
            dev_major = os.major(blk)
            dev_minor = os.minor(blk)
            dn = "{}:{}".format(dev_major, dev_minor)
            if dn in dev_cache:
                returndict[path] = dev_cache[dn]
            elif dn in devnum_cache:
                dev_cache[dn] = devnum_cache[dn]
                returndict[path] = dev_cache[dn]
            else:
                unknown_paths.append(path)
        except Exception as _:
            if sillyfacter.config.STRICT:
                raise
            if sillyfacter.config.DEBUG:
                dp = debugprint()
                dp("Error in stat of '{}' exception is '{}'".
                   format(path, _), "_linux_use_stat")
            if log:
                log.error("Stat failed for path: {}".format(path))
            returndict[path] = None
    if len(unknown_paths) > 0:
        unknown_cache = _run_df(unknown_paths, df_cmd=["/bin/df", "-P"])
        returndict = dict(returndict.items() + unknown_cache.items())
    return returndict


def _cat(f):
    l = []
    with open(f, "r") as g:
        for _ in g.readlines():
            l.append(_.strip())
    return l


def _devicenum_cache(log=None):
    dev_cache = {}
    #
    if os.path.exists("/proc/mounts"):
        for i in _cat("/proc/mounts"):
            mount_line = i.split()
            mount_device = mount_line[0]
            mount_point = mount_line[1]
            #mount_fstype = mount_line[2]
            try:
                blk = os.stat(mount_point)[stat.ST_DEV]
                dev_major = os.major(blk)
                dev_minor = os.minor(blk)
                dn = "{}:{}".format(dev_major, dev_minor)
                dev_cache[dn] = mount_device
            except Exception as _:
                if sillyfacter.config.STRICT:
                    raise
                if sillyfacter.config.DEBUG:
                    dp = debugprint()
                    dp("Error in stat of '{}' exception is '{}'".
                       format(mount_point, _), "_devicenum_cache")
                if log:
                    log.error("Stat failed for path: {} ({})".format
                              (mount_point, mount_device))
                pass
    #
    if os.path.exists("/proc/partitions"):
        for i in _cat("/proc/partitions"):
            if len(i) > 0 and re.match('\d+', i[0]):
                dev = i.split()
                dev_cache["{}:{}".format(dev[0], dev[1])] = "/dev/" + dev[-1]
    # this is never needed now
    if True is False and os.path.exists("/proc/fs/nfsfs/servers") and \
            os.path.exists("/proc/fs/nfsfs/volumes"):
        nfsfs_volumes = {}
        nfsfs_servers = {}
        for i in _cat("/proc/fs/nfsfs/volumes")[1:]:
            if len(i) > 0:
                dev_num = i.split()[3]
                server_id = i.split()[1]
            if re.match(".+:.+", dev_num):
                nfsfs_volumes[dev_num] = server_id
        for i in _cat("/proc/fs/nfsfs/servers")[1:]:
            if len(i) > 0:
                servername = i.split()[-1]
                server_id = i.split()[1]
            if re.match(".+:.+", dev_num):
                nfsfs_servers[server_id] = servername
        for dev_num in nfsfs_volumes:
            server_id = nfsfs_volumes[dev_num]
            try:
                dev_cache[dev_num] = "nfs://" + nfsfs_servers[server_id]
            except:
                if sillyfacter.config.STRICT:
                    raise
                pass
    #
    return dev_cache


def find_device_fstype(device):
    for p in psutil.disk_partitions(all=True):
        if (p.device == device):
            return p.fstype
    return "unknown"


def sanitize_conns(connection_pairs, nslookup):
    sanitized_pairs = []
    for connection_pair in connection_pairs:
        src_conn = connection_pair[0]
        dst_conn = connection_pair[1]
        try:
            src_ip = str(netaddr.IPAddress(src_conn[0]).ipv4())
        except:
            src_ip = str(netaddr.IPAddress(src_conn[0]))
        '''
        if src_ip not in nslookup:
            src_name = timeout(
                fetch_dns, (src_ip,), timeout_duration=5, default=src_ip)
            if src_name != src_ip:
                nslookup[src_ip] = src_name
        '''
        src_port = src_conn[1]
        src_socket = "%s:%s" % (src_ip, src_port)
        dst_socket = None
        if (len(dst_conn) > 1):
            try:
                dst_ip = str(netaddr.IPAddress(dst_conn[0]).ipv4())
            except:
                dst_ip = str(netaddr.IPAddress(dst_conn[0]))
            '''
            if dst_ip not in nslookup:
                dst_name = timeout(
                    fetch_dns, (dst_ip,), timeout_duration=5, default=dst_ip)
                if dst_name != dst_ip:
                    nslookup[dst_ip] = dst_name
            '''
            dst_port = dst_conn[1]
            dst_socket = "%s:%s" % (dst_ip, dst_port)
        sanitized_pairs.append([src_socket, dst_socket])
    return sanitized_pairs, nslookup






